unit uFuenteConstante;

interface

uses
  Classes, uFichasLPD, xMatDefs, uFechas,
  uCosa, uGlobs, uconstantesSimSEE,
  uFuentesAleatorias, SysUtils;

resourcestring
  rsFichaDe = 'Ficha de';
  rsFuenteConstante = 'Fuente constante';

type

(*+doc TFichaFuenteConstante
Ficha de parámetros dinámicos para la fuente constante.
Permite especificar el valor constante.
-doc*)

  { TFichaFuenteConstante }

  TFichaFuenteConstante = class(TFichaLPD)
  public
    valor: NReal;
    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      valor: NReal);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    class function DescClase: string; override;
    function infoAd_: string; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion:integer): TDataColumnListOfCosa; override;

  end;

(*+doc TFuenteConstante
Esta no es una fuente aleatoria propiamente dicha, pues su valor es "constante"
en el sentido que no sale de un sorteo. El valor puede ser variado con el servicio
de parámetros dinámicos.
-doc*)

  { TFuenteConstante }

  TFuenteConstante = class(TFuenteAleatoriaConFichas)
  public
    pa: TFichaFuenteConstante;

    constructor Create(capa: integer; nombre: string;
      xdurPasoDeSorteoEnHoras: integer; resumirPromediando: boolean; lpd: TFichasLPD);
    class function TipoFichaFuente: TClaseDeFichaLPD; override;
    //function InfoAd : String; override;
    class function DescClase: string; override;
    procedure PrepararMemoria(globs: TGlobs); override;
    procedure RegistrarParametrosDinamicos; override;
    procedure SortearEntradaRB(var aRB: NReal); override;
    procedure ValorEsperadoEntradaRB(var aRB: NReal); override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: integer=-2): TDataColumnListOfCosa; override;

  end;

procedure AlInicio;
procedure AlFinal;


implementation

//---------------------------------
// Métodos de TFichaFuenteConstante
//=================================

constructor TFichaFuenteConstante.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; valor: NReal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.valor := valor;
end;

constructor TFichaFuenteConstante.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('valor', valor);
  f.EjecutarLectura;
end;

procedure TFichaFuenteConstante.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('valor', valor, uconstantesSimSEE.CF_PRECISION, uconstantesSimSEE.CF_DECIMALES);
end;

class function TFichaFuenteConstante.DescClase: string;
begin
  Result := rsFichaDe + ' ' + rsFuenteConstante;
end;

function TFichaFuenteConstante.infoAd_: string;
begin
  Result := 'Valor= ' + FloatToStrF(valor, ffGeneral, 10, 2);
end;

class function TFichaFuenteConstante.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:= TDataColumnListOfCosa.Create(TFichaFuenteConstante,xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('valor', TFichaFuenteConstante(nil).valor, TDataTypeDoubleOfCosa, '', 113, -1);
  Result:=list;
end;

//----------------------------
// Métodos de TFuenteConstante
//============================

constructor TFuenteConstante.Create(capa: integer; nombre: string;
  xdurPasoDeSorteoEnHoras: integer; resumirPromediando: boolean; lpd: TFichasLPD);

begin
  inherited Create(
    capa,
    nombre,
    xdurPasoDeSorteoEnHoras,
    resumirPromediando,
    lpd);
  pa := nil;
end;

class function TFuenteConstante.TipoFichaFuente: TClaseDeFichaLPD;
begin
  Result := TFichaFuenteConstante;
end;

{function TFuenteConstante.infoAd : String;
begin
  result := 'Constante';
end;}

class function TFuenteConstante.DescClase: string;
begin
  Result := rsFuenteConstante;
end;

procedure TFuenteConstante.PrepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
end;

procedure TFuenteConstante.RegistrarParametrosDinamicos;
begin
  //Se llama solo despues de preparar memoria que ya expande las fichas
  //no hace falta volver a hacerlo y es un proceso caro si hay periodicidad
  //   lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pa,
    nil, cambioFichaPDFuenteAleatoria);
end;

procedure TFuenteConstante.SortearEntradaRB(var aRB: NReal);
begin
  if not entradasFijadas then
  begin
    aRB := pa.valor;
    entradasFijadas := True;
  end;
end;

procedure TFuenteConstante.ValorEsperadoEntradaRB(var aRB: NReal);
begin
  SortearEntradaRB(aRB);
end;

class function TFuenteConstante.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: integer): TDataColumnListOfCosa;
var
   list: TDataColumnListOfCosa;

begin
  //list:= TDataColumnListOfCosa.Create(TFuenteConstante, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  result:=list;

end;


procedure AlInicio;
begin
  registrarClaseDeCosa(TFuenteConstante.ClassName, TFuenteConstante);
  registrarClaseDeCosa(TFichaFuenteConstante.ClassName, TFichaFuenteConstante);
end;

procedure AlFinal;
begin
end;

end.
