unit uStringGridWithControls;

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Grids,
  StdCtrls, types, Buttons, ExtCtrls, uvisordetabla;

type


  { TSGWC_BitBtn }
  TSGWC_BitBtn = class(TBitBtn)

    procedure MouseWheelOnBitBtn (Sender: TObject; Shift: TShiftState;
         WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);

  public
    kRow: Integer;
    iid: Integer;
    constructor Create(TheOwner: TComponent; kRow: integer;
      iid: Integer; Hint: string='');

  end;

  //Tipo de dato de la columna
  TTipoColumna = (TC_Texto, TC_BitBtn, TC_CheckBox, TC_ComboBox, TC_BitBtns);
  TTipoColumnas = array of TTipoColumna;

  { TStringGridWithControls }

  TStringGridWithControls = class(TStringGrid)
  protected

    _maxTopRow:Integer;
    _tipoCols: TTipoColumnas;
    _maxHeight: Integer;
    _maxWidth:Integer;

    procedure TopLeftChanged(Sender: TObject);
    procedure MouseWheelOnSGWC (Sender: TObject; Shift: TShiftState;
         WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);

    function GetControlOnCell(kCol, kRow: integer): TControl;
    procedure SetControlOnCell(kCol, kRow: integer; AValue: TControl);

    property ControlOnCell[kCol, kRow: integer]: TControl
      read GetControlOnCell write SetControlOnCell;
    procedure MoveControl(AControl: TControl; NewPosition: TPoint;
      isVisible: boolean = True);

    procedure BitBtn_OnClick( Sender: TObject); virtual;
    procedure CheckBox_OnClick( Sender: TObject); virtual;

    procedure Borrar_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure BorrarDisabled_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Editar_onClick( nidRec: string; kFila: integer ); virtual;
    procedure Ver_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure VerNaranja_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Bajar_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure BajarDelTodo_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Configurar_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Email_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Info_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure EmailDisabled_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure SemaforoRojo_OnClick( nidRec: string; kFila: integer); virtual;
    procedure SemaforoVerde_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure SubirDelTodo_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Subir_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure Lupa_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure checkbox_0_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure checkbox_1_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure clonar_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure fdownload_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure fupload_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure radiobutton_0_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure radiobutton_1_OnClick( nidRec: string; kFila: integer ); virtual;
    procedure reset_OnClick( nidRec: string; kFila: integer ); virtual;

    procedure ReArrangeControls;

    function ajustarCentrado (R1, R2: TRect):Boolean;

  public
    { public declarations }

    ImageList: TImageList;
    constructor Create(AOwner: TComponent; AName: string);

    procedure SetEncabezados(encabezados: array of string;
      tipoColumnas: array of TTipoColumna);

    procedure InsetarFila;

    // Escribe el texto en la celda.
    procedure wrText(kCol, kRow: integer; Text: string);

    // Inserta TCheckBox en la celda kCol, kRow
    procedure wrCheckBox(kCol, kRow: integer; Checked: boolean;
      Hint: string = '');

    // Inserta TComboBox en la celda kCol, kRow
    procedure wrComboBox(kCol, kRow, Width: integer); virtual;

    // Escribe un array de BitBtn en la misma celda
    procedure wrBitBtns(kCol, kRow: integer;
      iids: array of integer; // ej [ iid_Borrar, iid_BorrarDisabled,  iid_Editar]
      hints: array of string);


    procedure AutoSizeColumns;
    procedure AutoSizeColumn(aCol: Integer);

    procedure Free;

  end;

implementation

{ TSGWC_BitBtn }

procedure TSGWC_BitBtn.MouseWheelOnBitBtn(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var
  sg:TStringGridWithControls;
begin
  sg:=TStringGridWithControls(TSGWC_BitBtn(sender).Owner);
  sg.MouseWheelOnSGWC(Sender, Shift, WheelDelta, MousePos, Handled);

end;

constructor TSGWC_BitBtn.Create(TheOwner: TComponent; kRow: integer;
  iid: Integer; Hint: string);
begin

  Inherited Create(TheOwner);
  self.kRow:= kRow;
  self.iid:=iid;
  if Hint<>'' then
    self.Hint:= Hint
  else
    self.Hint:=hintPorDefecto(iid);

  self.Caption:='';
  self.OnMouseWheel:=MouseWheelOnBitBtn;

end;

{ TStringGridWithControls }

procedure TStringGridWithControls.TopLeftChanged(Sender: TObject);
begin
  reArrangeControls;
end;

function TStringGridWithControls.GetControlOnCell(kCol, kRow: integer): TControl;
begin
  Result := TControl(Objects[kCol, kRow]);
end;

procedure TStringGridWithControls.SetControlOnCell(kCol, kRow: integer;
  AValue: TControl);
begin
  Objects[kCol, kRow] := AValue;
end;

procedure TStringGridWithControls.MoveControl(AControl: TControl;
  NewPosition: TPoint; isVisible: boolean);
var
  Width, Height: integer;
begin
  AControl.Visible := isVisible;
  Width := AControl.Width;
  Height := AControl.Height;
  AControl.SetBounds(NewPosition.x, NewPosition.y, Width, Height);
end;

procedure TStringGridWithControls.BitBtn_OnClick( Sender: TObject);
var
  btn:TSGWC_BitBtn;
  nidRec: String;
  tipoBoton: Integer;
  kFila: integer;
begin
  btn:= TSGWC_BitBtn(Sender);
  kFila:=btn.kRow;
  tipoBoton:= btn.iid;
  case tipoBoton of
    iid_Borrar: Borrar_OnClick( nidRec, kFila );
    iid_BorrarDisabled: BorrarDisabled_OnClick( nidRec, kFila );
    iid_Editar: Editar_OnClick( nidRec, kFila );
    iid_Ver: Ver_OnClick( nidRec, kFila );
    iid_VerNaranja: VerNaranja_OnClick( nidRec, kFila );
    iid_Bajar: Bajar_OnClick( nidRec, kFila );
    iid_BajarDelTodo: BajarDelTodo_OnClick( nidRec, kFila );
    iid_Configurar: Configurar_OnClick( nidRec, kFila );
    iid_Email: Email_OnClick( nidRec, kFila );
    iid_Info: Info_OnClick( nidRec, kFila );
    iid_EmailDisabled: EmailDisabled_OnClick( nidRec, kFila );
    iid_SemaforoRojo: SemaforoRojo_OnClick( nidRec, kFila );
    iid_SemaforoVerde: SemaforoVerde_OnClick( nidRec, kFila );
    iid_SubirDelTodo: SubirDelTodo_OnClick( nidRec, kFila );
    iid_Subir: Subir_OnClick( nidRec, kFila );
    iid_Lupa: Lupa_OnClick( nidRec, kFila );
    iid_checkbox_0: checkbox_0_OnClick( nidRec, kFila );
    iid_checkbox_1: checkbox_1_OnClick( nidRec, kFila );
    iid_clonar: clonar_OnClick( nidRec, kFila );
    iid_fdownload: fdownload_OnClick( nidRec, kFila );
    iid_fupload: fupload_OnClick( nidRec, kFila );
    iid_radiobutton_0: radiobutton_0_OnClick( nidRec, kFila );
    iid_radiobutton_1: radiobutton_1_OnClick( nidRec, kFila );
    iid_Reset: reset_OnClick( nidRec, kFila );
  end;
end;

procedure TStringGridWithControls.CheckBox_OnClick( Sender: TObject);
begin
  showMessage( 'Click on: '+ TComponent( Sender ).Name );
end;

procedure TStringGridWithControls.Borrar_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Borrar_OnClick Sin escribir..' + nidRec );
end;

procedure TStringGridWithControls.BorrarDisabled_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'BorrarDisabled_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Editar_onClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Editar_onClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Ver_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Ver_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.VerNaranja_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'VerNaranja_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Bajar_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Bajar_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.BajarDelTodo_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'BajarDelTodo_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Configurar_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Configurar_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Email_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Email_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Info_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Info_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.EmailDisabled_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'EmailDisabled_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.SemaforoRojo_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'SemaforoRojo_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.SemaforoVerde_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'SemaforoVerde_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.SubirDelTodo_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'SubirDelTodo_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Subir_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Subir_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.Lupa_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'Lupa_OnClick..' + nidRec );
end;
procedure TStringGridWithControls.checkbox_0_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'checkbox_0_OnClick..' + nidRec );
end;
procedure TStringGridWithControls.checkbox_1_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'checkbox_1_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.clonar_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'clonar_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.fdownload_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'fdownload_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.fupload_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'fupload_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.radiobutton_0_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'radiobutton_0_OnClick Sin escribir..' + nidRec );
end;
procedure TStringGridWithControls.radiobutton_1_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'radiobutton_1_OnClick Sin escribir.. ' + nidRec );
end;
procedure TStringGridWithControls.reset_OnClick( nidRec: string; kFila: integer );
begin
  showmessage( 'reset_OnClick Sin escribir.. ' + nidRec );
end;


procedure TStringGridWithControls.ReArrangeControls;
var
  kCol, kRow: integer;
  AControl: TControl;
begin

  _maxTopRow:=Self.RowCount-Self.VisibleRowCount;
  for kCol := 0 to ColCount - 1 do
    for kRow := 0 to RowCount - 1 do
    begin
      AControl := ControlOnCell[kCol, kRow];
      if AControl <> nil then
        begin
          MoveControl(AControl, CellRect(kCol, kRow).TopLeft,
            ((LeftCol <= kCol) and (TopRow <= kRow)));
        end;
    end;
end;

function TStringGridWithControls.ajustarCentrado(R1, R2: TRect): Boolean;
begin

end;

procedure TStringGridWithControls.Free;
var
  kCol, kRow: integer;
  AControl: TControl;
begin
  for kCol := 0 to ColCount - 1 do
    for kRow := 0 to RowCount - 1 do
    begin
      AControl := ControlOnCell[kCol, kRow];
      if AControl <> nil then
        AControl.Free;
    end;
  inherited Free;
end;

constructor TStringGridWithControls.Create(AOwner: TComponent; AName: string);
begin

  inherited Create(AOwner);

  self.Name := AName;
  self.ScrollBars := ssBoth;
  self.DefaultRowHeight:=25;

  self.Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine];

  self.FixedCols:=0;
  self.RowCount:=1;
  self.ColCount:=1;

  self.TopLeftChanged(self);
  self.OnTopLeftChanged := self.TopLeftChanged;
  self.OnMouseWheel:=self.MouseWheelOnSGWC;

  _maxTopRow:=0;

end;

procedure TStringGridWithControls.SetEncabezados(encabezados: array of string;
  tipoColumnas: array of TTipoColumna);
var
  nCols: Integer;
  i: Integer;
  gct: TGridColumnTitle;
  gc: TGridColumn;
  ntc: Integer;
begin

  nCols:=Length(encabezados);

  SetLength(self._tipoCols, nCols);
  self.ColCount:=nCols;

  ntc:=Length(tipoColumnas);
  for i:=0 to nCols -1 do
  begin
    if i<ntc then
      _tipoCols[i]:=tipoColumnas[i]
    else
      _tipoCols[i]:=TC_Texto;

    gc:=self.Columns.Add;
    gc.Title.Caption:=encabezados[i];

  end;
end;

procedure TStringGridWithControls.InsetarFila;
var
  i: Integer;
begin

  self.RowCount:=self.RowCount+1;
  for i:=0 to ColCount-1 do
  begin
    case _tipoCols[i] of
    TC_BitBtn:
      begin
        //wrBitBtn(i,RowCount-1,TSGWC_ButtonAction.SHOW);
      end;
    end;


  end;
end;

procedure TStringGridWithControls.MouseWheelOnSGWC(Sender: TObject;
  Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean);
begin
  if WheelDelta < 0 then
  begin
    writeln(TopRow);
    if TopRow < _maxTopRow then
      TopRow:=TopRow+1;
  end
  else
  begin
    if TopRow>1 then
      TopRow:=TopRow-1;
  end;
end;

procedure TStringGridWithControls.wrText(kCol, kRow: integer; Text: string);
begin
  self.Cells[kCol, kRow] := Text;
end;

procedure TStringGridWithControls.wrCheckBox(kCol, kRow: integer;
  Checked: boolean; Hint: string);
var
  R: Classes.TRect;
  P: TPanel;
  cb: TCheckBox;
  i, j: Integer;
begin

  R := self.CellRect(kCol, kRow);
  P:=TPanel.Create(self);
  P.BoundsRect:=R;
  P.Parent:=self;
  P.Color:=clWhite;

  P.Width:=(R.Bottom - R.Top)+2;

  cb := TCheckBox.Create(self);
  cb.Parent := P;
  cb.Caption:='';

  i:=trunc(P.Width/2);
  j:=trunc(P.Height/2);

  cb.Left := i - trunc(cb.Width/2)+3;
  cb.Top := j - trunc(cb.Height/2);

  ControlOnCell[kCol, kRow] := P;

end;

procedure TStringGridWithControls.wrComboBox(kCol, kRow, Width: integer);
begin

end;

procedure TStringGridWithControls.wrBitBtns(kCol, kRow: integer;
  iids: array of integer; hints: array of string);
var
  R: Classes.TRect;
  P: TPanel;
  Btn: TSGWC_BitBtn;
  i: Integer;
  nBtns:Integer;

begin

  R := self.CellRect(kCol, kRow);
  P:=TPanel.Create(self);
  P.BoundsRect:=R;
  P.Parent:=self;
  P.Color:=clBlack;

  nBtns:=Length(iids);
  P.Width:=nBtns*(R.Bottom - R.Top)+2;

  for i:=nBtns-1 downto 0 do
  begin
    if length(hints)>i then
      Btn := TSGWC_BitBtn.Create(self, kRow, iids[i], hints[i])
    else
      Btn := TSGWC_BitBtn.Create(self, kRow, iids[i]);

    Btn.Height:=R.Bottom - R.Top;
    Btn.Width:=R.Bottom - R.Top;
    Btn.Parent := P;
    Btn.OnClick := BitBtn_OnClick;
    Btn.Align:=alLeft;
    //TDataModule2.ImageList1.GetBitmap( iids[i],  Btn.Glyph );
    Btn.Spacing := 1;

  end;


  ControlOnCell[kCol, kRow] := P;

end;

procedure TStringGridWithControls.AutoSizeColumns;
var
  i: Integer;
begin
  for i:=0 to ColCount-1 do
    if _tipoCols[i] = TC_Texto then
      inherited AutoAdjustColumn(i)
    else
      AutoSizeColumn(i);

  ReArrangeControls;
end;

procedure TStringGridWithControls.AutoSizeColumn(aCol: Integer);
var
  maxHeight, maxWidth:integer;
  iRow: Integer;
  ctrl: TControl;
  theCol: TGridColumn;
begin
  maxHeight:=0;
  maxWidth:=0;
  for iRow:=0 to self.RowCount-1 do
  begin
    ctrl:=ControlOnCell[aCol, iRow];
    if Assigned(ctrl) then
    begin
      if ctrl.Width>maxWidth then
        maxWidth:=ctrl.Width;
      if ctrl.Height>maxHeight then
        maxHeight:=ctrl.Height;
    end;
  end;

  theCol := ColumnFromGridColumn(aCol);
  theCol.Width:=maxWidth+1;
end;


end.
